@@ -23,7 +23,7 @@ sub routes {
         },
         {   route  => '/:language/:controller',
             via    => [qw(GET OPTIONS)],
-            params => {$$config{language_param} => qr/(?:$l)/},
+            params => {$$config{language_param} => qr/(?:$l)/, controller => qr/[\w-]{3,}/},
             to     => {
 
                 #Ado::Control::Default
@@ -33,8 +33,12 @@ sub routes {
         },
         {   route  => '/:language/:controller/:action',
             via    => [qw(GET POST OPTIONS)],
-            params => {$$config{language_param} => qr/(?:$l)/},
-            to     => {
+            params => {
+                $$config{language_param} => qr/(?:$l)/,
+                controller               => qr/[\w-]{3,}/,
+                action                   => qr/\w{3,}/
+            },
+            to => {
 
                 #Ado::Control::Default
                 controller => 'Default',
@@ -43,8 +47,13 @@ sub routes {
         },
         {   route  => '/:language/:controller/:action/:id',
             via    => [qw(GET PUT DELETE OPTIONS)],
-            params => {$$config{language_param} => qr/($l)/},
-            to     => {
+            params => {
+                $$config{language_param} => qr/(?:$l)/,
+                controller               => qr/[\w-]{3,}/,
+                action                   => qr/\w{3,}/,
+                id                       => qr/\d+/
+            },
+            to => {
 
                 #Ado::Control::Default
                 controller => 'Default',
